// War170API.cpp
// Copyright 2000 Jarle (jgaa) Aase
// All rights reserved!

#include "war.h"
#include "War170API.h"
#include "PathList.h"

using namespace std;


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    int ErrorValue = 0;

    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }

    return TRUE;
}

static int TranslateError(const usrstat Err, const int Default);
static int TranslateError(const usrstat Err, const int Default)
{
    int Rval = WARAPI_ERR_UNDEFINED;
    switch(Err)
    {
    case USRERR_PERM:
        Rval = WARAPI_ERR_PERM;
        break;
    case USRERR_COMMFAIL:
        Rval = WARAPI_ERR_COMMFAIL;
        break;
    case USRERR_NOENT:
        Rval = WARAPI_ERR_NOT_FOUND;
        break;
    case USRERR_EXIST:
        Rval = WARAPI_ERR_USER_EXIST;
    }

    return Rval;
}

WAR170API_API WARAPI_RESULT 
Warapi_OpenSession(const WAR_OUT WARAPI_CCSTR_T Host, 
                   const WAR_OUT WARAPI_PORT Port,
                   const WAR_OUT WARAPI_CCSTR_T LoginName,
                   const WAR_OUT WARAPI_CCSTR_T Password,
                   WAR_IN WARAPI_SESSION_HANDLE *SessionHandle,
                   WAR_IN int *ErrorValue)
{
    WarString MyHost = Host;
    int Err = 0;

    if (!ErrorValue 
        || !Host
        || !*Host
        || !LoginName
        || !*LoginName
        || !Password
        || !*Password
        || !SessionHandle)
    {
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    *SessionHandle = NULL;

    WarUserHandle *pSession = new WarUserHandle;
    if (Port != 0)
    {
        MyHost << ':';
        MyHost << Port;
    }

    if (!pSession->Create(NULL, MyHost))
    {
        switch(pSession->GetLastError())
        {
        case USRERR_COMMFAIL:
            *ErrorValue = WARAPI_ERR_COMMFAIL;
            break;
        }
        goto failure;
    }

    while(!pSession->RPCLogin(LoginName, Password, &Err))
    {
        if ((pSession->GetLastError() == USRERR_PERM) 
            && (Err == LF_WAIT_FOR_DNS_LOOKUP))
        {
            ::Sleep(500);
            continue;
        }
        
        switch(pSession->GetLastError())
        {
        case 1:
            *ErrorValue = WARAPI_ERR_AUTH;
            break;
        }
        goto failure;
    }

    /* Logged on OK */

    *SessionHandle = (WARAPI_SESSION_HANDLE)pSession;
    *ErrorValue = WARAPI_ERR_OK;
    return WARAPI_CALL_OK;

failure:
    if (pSession)
        delete pSession;
    return WARAPI_CALL_GENERIC_FAIL;
}

WAR170API_API WARAPI_RESULT 
Warapi_ReleaseUserHandle(WARAPI_USER_HANDLE Handle)
{
    if (!Handle)
        return WARAPI_CALL_EINVAL;

    user *p = (user *)Handle;
    delete Handle;

    return WARAPI_CALL_OK;
}

WAR170API_API WARAPI_RESULT 
Warapi_LookupUser(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WAR_OUT WARAPI_CCSTR_T Name,
                  const WAR_OUT WARAPI_SEARCH_MASK SearchMask,
                  WAR_IN WARAPI_USER_HANDLE *UserHandle,
                  WAR_IN int *ErrorValue)
{
    if (!ErrorValue 
        || !SessionHandle
        || !Name
        || !*Name
        || !UserHandle)
    {
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    *UserHandle = NULL;
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;
    
    PathList Path = Name;
    if (!Path.empty() && (!strcmp(Path.front().c_str(), "system")))
        Path.pop_front(); // Remove "system"
    else 
        goto bad_path;
    if (Path.empty())
    {
bad_path:
        *ErrorValue = WARAPI_ERR_INVALID_GROUPPATH;
        return WARAPI_CALL_GENERIC_FAIL;
    }
    
    user_attrs Attrs;
    memset(&Attrs, 0, sizeof(Attrs));
    user SearchNode = USER_SYSTEM;
    
    while(!Path.empty())
    {   
        if (!pHandle->FindUser(SearchNode, UT_NODEDEFAULT, Path.front().c_str(), 
            UF_FIRST, Attrs))
        {
            *ErrorValue = TranslateError(pHandle->GetLastError(), 
                WARAPI_ERR_UNDEFINED_SEARCHERR);
            
            return WARAPI_CALL_GENERIC_FAIL;
        }
        SearchNode = Attrs.m_User;
        Path.pop_front();
    }


    if (!(((SearchMask & WARAPI_SEARCH_MASK_DOMAIN) && (Attrs.m_Type == UT_DOMAIN))
        || ((SearchMask & WARAPI_SEARCH_MASK_CLASS) && (Attrs.m_Type == UT_CLASS))
        || ((SearchMask & WARAPI_SEARCH_MASK_GROUP) && (Attrs.m_Type == UT_GROUP))
        || ((SearchMask & WARAPI_SEARCH_MASK_USER) && (Attrs.m_Type == UT_USER))
        || ((SearchMask & WARAPI_SEARCH_MASK_ALIAS) && (Attrs.m_Type == UT_ALIAS))))    
    {
        *ErrorValue = WARAPI_ERR_SRCMASK_MISMATCH;
        return WARAPI_CALL_GENERIC_FAIL;
    }

    *UserHandle = (WARAPI_USER_HANDLE)(user *) new user(Attrs.m_User);
    *ErrorValue = WARAPI_ERR_OK;
    return WARAPI_CALL_OK;
}

WAR170API_API WARAPI_RESULT 
Warapi_CreateUser(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WAR_OUT WARAPI_CCSTR_T Parent, 
                  const WAR_OUT WARAPI_CCSTR_T Name,
                  WAR_IN WARAPI_USER_HANDLE *UserHandle,
                  WAR_IN int *ErrorValue)
{
    if (!ErrorValue 
        || !SessionHandle
        || !Parent
        || !*Parent
        || !Name
        || !*Name
        || !UserHandle)
    {
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    *UserHandle = NULL;
    
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;

    WARAPI_USER_HANDLE hParent;

    if (Warapi_LookupUser(SessionHandle, 
            Parent, 
            WARAPI_SEARCH_MASK_CLASS | WARAPI_SEARCH_MASK_GROUP,
            &hParent, 
            ErrorValue) != WARAPI_CALL_OK)
        return WARAPI_CALL_GENERIC_FAIL;
    
    user NewID;
    user_create_req Req;
    memset(&Req, 0, sizeof(Req));
    
    Req.m_Type = UT_USER;
    Req.m_Name = (RPC_userid)Name;
    Req.m_Father = (*(user *)hParent);
    Req.m_Owner_uid = USER_INVALID;
    Req.m_Owner_gid = USER_INVALID;
    Req.m_Mode = 744;
    memset(&Req.m_Groups, 0, sizeof(user_gids));

    Warapi_ReleaseUserHandle(hParent);
    
    if (!pHandle->CreateNew(Req, NewID))
    {
        usrstat Err = pHandle->GetLastError();
        *ErrorValue = TranslateError(Err, WARAPI_ERR_CREATE_FAILED_UNDEFINED);
        return WARAPI_CALL_GENERIC_FAIL;
    }
    
    *UserHandle = (WARAPI_USER_HANDLE)(user *) new user(NewID);
    
    return WARAPI_CALL_OK;
}

WAR170API_API WARAPI_RESULT 
Warapi_DeleteUser(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WAR_OUT WARAPI_CCSTR_T Name,
                  WAR_IN int *ErrorValue)
{
     if (!ErrorValue 
        || !SessionHandle
        || !Name
        || !*Name)
    {
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;

    WARAPI_USER_HANDLE hNode;

    if (Warapi_LookupUser(SessionHandle, 
        Name, 
        WARAPI_SEARCH_MASK_USER,
        &hNode, 
        ErrorValue) != WARAPI_CALL_OK)
        return WARAPI_CALL_GENERIC_FAIL;

    if (!pHandle->DeleteNode(*(user *)hNode))
    {
        usrstat Err = pHandle->GetLastError();
        *ErrorValue = TranslateError(Err, WARAPI_ERR_CREATE_FAILED_UNDEFINED);
        return WARAPI_CALL_GENERIC_FAIL;
    }

    *ErrorValue = WARAPI_ERR_OK;
    return WARAPI_CALL_OK;
}

WAR170API_API WARAPI_RESULT 
Warapi_CloseSession(const WAR_OUT WARAPI_SESSION_HANDLE SessionHandle)
{
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;

    if (!pHandle)
        return WARAPI_CALL_EINVAL;

    pHandle->RPCLogout();

    delete pHandle;

    return WARAPI_CALL_OK;
}

WAR170API_API WARAPI_RESULT 
Warapi_SetPassword(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  WAR_OUT WARAPI_USER_HANDLE UserHandle,
                  const WAR_OUT WARAPI_CCSTR_T Password,
                  WAR_IN int *ErrorValue)
{
     if (!ErrorValue 
        || !SessionHandle
        || !Password)
    {
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;

    if (!pHandle->SetPwd(*(user *)UserHandle, Password))
    {
        usrstat Err = pHandle->GetLastError();
        *ErrorValue = TranslateError(Err, WARAPI_ERR_UNDEFINED);
        return WARAPI_CALL_GENERIC_FAIL;
    }
    
    *ErrorValue = WARAPI_ERR_OK;
    return WARAPI_CALL_OK;
}

WAR170API_API WARAPI_RESULT 
Warapi_SetPath(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  WAR_OUT WARAPI_USER_HANDLE UserHandle,
                  const WAR_OUT WARAPI_CCSTR_T VirtualPath,
                  const WAR_OUT WARAPI_CCSTR_T PhysPath,
                  const WAR_OUT WARAPI_FLAGS Flags,
                  WAR_IN int *ErrorValue)
{
    if (!ErrorValue 
        || !PhysPath
        || !*PhysPath
        || !SessionHandle
        || !Flags)
    {
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;
    WARAPI_CCSTR_T MyVirtualPath = VirtualPath;

    DWORD MyFlags = 0;
    if (Flags & WARAPI_PERM_READ) MyFlags |= AF_READ;
    if (Flags & WARAPI_PERM_WRITE) MyFlags |= AF_WRITE;
    if (Flags & WARAPI_PERM_EXECUTE) MyFlags |= AF_EXECUTE;
    if (Flags & WARAPI_PERM_LIST) MyFlags |= AF_LIST;
    if (Flags & WARAPI_PERM_DELETE) MyFlags |= AF_DELETE;
    if (Flags & WARAPI_PERM_REMOVE) MyFlags |= AF_REMOVE;
    if (Flags & WARAPI_PERM_CREATE) MyFlags |= AF_CREATE;
    if (Flags & WARAPI_PERM_HIDE) MyFlags |= AF_HIDE;
    if (Flags & WARAPI_PERM_RECURSIVE) MyFlags |= AF_RECURSIVE;
    if (Flags & WARAPI_PERM_DENY) MyFlags |= AF_DENY;
    if (Flags & WARAPI_PERM_FREE) MyFlags |= AF_FREE;

    if (!MyVirtualPath)
        MyVirtualPath = "";

    if (*MyVirtualPath)
        MyFlags |= AF_MAP;

    string MyPath;
    if (strnicmp(PhysPath, "file://", 7)
        && strnicmp(PhysPath, "vfsys://", 8))
        MyPath = "file://";
    MyPath += PhysPath;

    if (!pHandle->SetFileAttrs(*(user *)UserHandle, 
        MyPath.c_str(), 
        MyVirtualPath, 
        0, 
        0xffff,
        MyFlags))
    {
        usrstat Err = pHandle->GetLastError();
        *ErrorValue = TranslateError(Err, WARAPI_ERR_UNDEFINED);
        return WARAPI_CALL_GENERIC_FAIL;
    }
    
    *ErrorValue = WARAPI_ERR_OK;
    return WARAPI_CALL_OK;
}

WAR170API_API WARAPI_RESULT 
Warapi_Shutdown(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WARAPI_BOOL_T WaitForUsersToFinish,
                  WAR_IN int *ErrorValue)
{
    if (!ErrorValue 
        || ((WaitForUsersToFinish != 1) && (WaitForUsersToFinish != 0))
        || !SessionHandle)
    {
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    DWORD Flag = 0;

    if (WaitForUsersToFinish)
        Flag = SMF_LogoffPendingShutdown;
    else
        Flag = SMF_LogoffPendingShutdown | SMF_LogoffASAP;

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;

    if (!pHandle->SesMgrSetFlag(Flag, Flag,SSFM_GLOBAL))
    {
        usrstat Err = pHandle->GetLastError();
        *ErrorValue = TranslateError(Err, WARAPI_ERR_UNDEFINED);
        return WARAPI_CALL_GENERIC_FAIL;
    }

    *ErrorValue = WARAPI_ERR_OK;
    return WARAPI_CALL_OK;
}


WAR170API_API WARAPI_RESULT 
Warapi_SetLoginPermission(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  WAR_OUT WARAPI_USER_HANDLE UserHandle,
                  const WAR_OUT WARAPI_LOGIN_TYPE Type,
                  const WAR_OUT WARAPI_LOGIN_PERM Permission, 
                  WAR_IN int *ErrorValue)
{
    user_access_type AccessType;
    user_login_access LoginAccess;

    if (!ErrorValue 
        || !UserHandle
        || !SessionHandle)
    {
einval:
        if (ErrorValue)
            *ErrorValue = WARAPI_ERR_EINVAL;
        return WARAPI_CALL_EINVAL;
    }

    switch(Type)
    {
    case WARAPI_LOGIN_TYPE_FTPUSER:
        AccessType = UAT_FTP;
        break;
    case WARAPI_LOGIN_TYPE_ADMIN:
        AccessType = UAT_USERDB_ADMIN;
        break;
    default:
        goto einval;
    }

    switch(Permission)
    {
    case WARAPI_LOGIN_PERM_DENY:
        LoginAccess = ULA_DENY;
        break;
    case WARAPI_LOGIN_PERM_ALLOW:
        LoginAccess = ULA_ALLOW;
        break;
    case WARAPI_LOGIN_PERM_DEFAULT:
        LoginAccess = ULA_DEFAULT;
        break;
     default:
        goto einval;
    }

    *ErrorValue = WARAPI_ERR_UNDEFINED;
    WarUserHandle *pHandle = (WarUserHandle *)SessionHandle;

    if (!pHandle->SetAccess(*(user *)UserHandle, AccessType, LoginAccess))
    {
        usrstat Err = pHandle->GetLastError();
        *ErrorValue = TranslateError(Err, WARAPI_ERR_UNDEFINED);
        return WARAPI_CALL_GENERIC_FAIL;
    }
    
    *ErrorValue = WARAPI_ERR_OK;
    return WARAPI_CALL_OK;
}

